//Library scripts for spawning

void spawn01(void vName, float fX, float fY, float fZ)
{
	//spawn01 (Generic spawner)
	//Damon Vaughn Caskey
	//07/06/2007
	//
	//Spawns entity next to caller.
	//
	//vName: Model name of entity to be spawned in.
	//fX: X location adjustment.
	//fY: Y location adjustment.
      //fZ: Z location adjustment.

	void self = getlocalvar("self"); //Get calling entity.
	void vSpawn; //Spawn object.
	int  iDirection = getentityproperty(self, "direction");

	clearspawnentry(); //Clear current spawn entry.
      setspawnentry("name", vName); //Acquire spawn entity by name.

	if (iDirection == 0){ //Is entity facing left?                  
          fX = -fX; //Reverse X direction to match facing.
	}

      fX = fX + getentityproperty(self, "x"); //Get X location and add adjustment.
      fY = fY + getentityproperty(self, "a"); //Get Y location and add adjustment.
      fZ = fZ + getentityproperty(self, "z"); //Get Z location and add adjustment.
	
	vSpawn = spawn(); //Spawn in entity.

	changeentityproperty(vSpawn, "position", fX, fZ, fY); //Set spawn location.
	changeentityproperty(vSpawn, "direction", iDirection); //Set direction.
    
	return vSpawn; //Return spawn.
}

void spawn02(void vName, float fX, float fY, float fZ)
{
	//Spawns entity based on left screen edge
	//
	//vName: Model name of entity to be spawned in.
	//fX: X distance relative to left edge
	//fY: Y height from ground
      //fZ: Z location adjustment

	void self = getlocalvar("self"); //Get calling entity.
	void vSpawn; //Spawn object.
	int Direction = getentityproperty(self, "direction");
        int XPos = openborvariant("xpos"); //Get screen edge's position
        int Screen = openborvariant("hResolution"); // Get screen width

	clearspawnentry(); //Clear current spawn entry.
      setspawnentry("name", vName); //Acquire spawn entity by name.

   if (Direction == 0){ //Is entity facing left?                  
      fX = Screen-fX; //Reverse X direction to match facing and screen length
   }

      fZ = fZ + getentityproperty(self, "z"); //Get Z location and add adjustment.

	vSpawn = spawn(); //Spawn in entity.

	changeentityproperty(vSpawn, "position", fX + XPos, fZ, fY); //Set spawn location.
	return vSpawn; //Return spawn.
}

void spawn03(void vName, float fX, float fY, float fZ, int Dir)
{
	//spawn01 (Generic spawner)
	//Damon Vaughn Caskey
	//07/06/2007
	//
	//Spawns entity next to caller.
	//
	//vName: Model name of entity to be spawned in.
	//fX: X location adjustment.
	//fY: Y location adjustment.
        //fZ: Z location adjustment.
	//Dir: Direction setting

	void self = getlocalvar("self"); //Get calling entity.
	void vSpawn; //Spawn object.
	int  iDirection = getentityproperty(self, "direction");

	clearspawnentry(); //Clear current spawn entry.
      setspawnentry("name", vName); //Acquire spawn entity by name.

	if (iDirection == 0){ //Is entity facing left?                  
          fX = -fX; //Reverse X direction to match facing.
	}

      fX = fX + getentityproperty(self, "x"); //Get X location and add adjustment.
      fY = fY + getentityproperty(self, "a"); //Get Y location and add adjustment.
      fZ = fZ + getentityproperty(self, "z"); //Get Z location and add adjustment.
	
	vSpawn = spawn(); //Spawn in entity.

	changeentityproperty(vSpawn, "position", fX, fZ, fY); //Set spawn location.
	changeentityproperty(vSpawn, "direction", Dir); //Set direction.
    
	return vSpawn; //Return spawn.
}

void spawn06(void vName, float fX, float fY, float fZ)
{
	//Spawns entity based on left screen edge and z axis
	//Auto adjust with camera's position
	//vName: Model name of entity to be spawned in.
	//fX: X distance relative to left edge
	//fY: Y height from ground
      //fZ: Z coordinate

	void self = getlocalvar("self"); //Get calling entity.
	void vSpawn; //Spawn object.
	int Direction = getentityproperty(self, "direction");
        int XPos = openborvariant("xpos"); //Get screen edge's position
        int YPos = openborvariant("ypos"); // Get camera position
        int Screen = openborvariant("hResolution"); // Get screen width

	clearspawnentry(); //Clear current spawn entry.
      setspawnentry("name", vName); //Acquire spawn entity by name.

   if (Direction == 0){ //Is entity facing left?                  
      fX = Screen-fX; //Reverse X direction to match facing and screen length
   }

	vSpawn = spawn(); //Spawn in entity.

	changeentityproperty(vSpawn, "position", fX + XPos, fZ + YPos, fY); //Set spawn location.
	return vSpawn; //Return spawn
}

void spawnXT(void vName, float fX, float fY, float fZ, void Code, int Value)
{//Spawns entity with spawn06 only if defined global variable is not set
 //vName: Model name of entity to be spawned in
 //fX: X location adjustment
 //fY: Y location adjustment
 //fZ: Z location adjustment
 //Code: Global variable to check
 //Value: Value to compare
    int K = getglobalvar(Code);

    if(K!=Value){
      spawn06(vName, fX, fY, fZ);
      setglobalvar(Code, Value);
    }
}

void spawnAt(void Name, float dx, float dy, float dz)
{ // Spawn an entity and place it at exact coords
   void self = getlocalvar("self");
   void Spawn;

   Spawn = spawn01(Name, 0, 0, 0);
   changeentityproperty(Spawn, "position", dx, dy, dz);
}

void spawner(void vName, float fX, float fY, float fZ)
{	//Spawns entity next to caller and set them as child.
	//
	//vName: Model name of entity to be spawned in.
	//fX: X location adjustment.
	//fY: Y location adjustment.
      //fZ: Z location adjustment.

	void self = getlocalvar("self"); //Get calling entity.
	void vSpawn; //Spawn object.
	
	vSpawn = spawn01(vName, fX, fY, fZ); //Spawn in entity.

	changeentityproperty(vSpawn, "parent", self); //Set caller as parent.
    
	return vSpawn; //Return spawn.
}

void shooter(void Shot, float dx, float dy, float dz, float Vx, float Vy, float Vz)
{ // Shooting projectile with speed control
   void self = getlocalvar("self");
   int Direction = getentityproperty(self, "direction");
   int x = getentityproperty(self, "x");
   int y = getentityproperty(self, "a");
   int z = getentityproperty(self, "z");
   void vShot;

   if (Direction == 0){ //Is entity facing left?                  
      dx = -dx; //Reverse X direction to match facing
   }

   vShot = projectile(Shot, x+dx, z+dz, y+dy, Direction, 1, 0, 0);
   changeentityproperty(vShot, "velocity", Vx, Vz, Vy);
   changeentityproperty(vShot, "speed", Vx);
   }

void shooter2(void Shot, float dx, float dy, float Vx, float Vy)
{ // Shooting special projectile with speed control
   void self = getlocalvar("self");
   int Direction = getentityproperty(self, "direction");
   void vShot;

   if (Direction == 0){ //Is entity facing left?                  
      Vx = -Vx; //Reverse Vx direction to match facing
   }

   vShot = spawn01(Shot, dx, dy, 0);
   changeentityproperty(vShot, "velocity", Vx, 0, Vy);
   changeentityproperty(vShot, "owner", self);
   return vShot;
}

void shooterM(void Shot, float dx, float dy, float Vx, float Vy)
{ // Shooting special projectile with speed control and matches its map with own's map
   void self = getlocalvar("self");
   int map = getentityproperty(self,"map");
   void Spawn;

   Spawn = shooter(Shot, dx, dy, Vx, Vy, 1);

   if(map != 1){
     changeentityproperty(Spawn, "map", map);
   }
}

void shooterAngle(void Shot, float dx, float dy, float V, int Angle)
{ // Shooting special projectile with speed control at defined Angle
   void self = getlocalvar("self");
   int Direction = getentityproperty(self, "direction");
   int Vx = V*cos(Angle);
   int Vy = 2*V*sin(Angle);
   void vShot;

   vShot = shooter(Shot, dx, dy, Vx, Vy, 1);

   if (Direction == 1){
     Angle = 360-Angle;
   }

   if(Angle >= 360){
     Angle = Angle-360;
   } else if(Angle < 0){
     Angle = Angle+360;
   }

   changedrawmethod(vShot, "rotate", Angle);
   return vShot;
}

void spawnBind(void Name, float dx, float dy, float dz, int Dir, int Flag, int Num)
{ // Spawn entity, bind it and store each other
   void self = getlocalvar("self");
   void Spawn;

   Spawn = spawn01(Name, dx, dy, dz);
   bindentity(Spawn, self, dx, dz, dy, Dir, Flag);
   setentityvar(self, Num, Spawn);
   setentityvar(Spawn, Num, self);

   return Spawn; //Return spawn
}

void spawnBA(void Shot, float dx, float dy, float dz, int Dir, int Flag, int Angle)
{// Spawn entity at defined Angle and bind it
   void self = getlocalvar("self");
   void Spawn;

   Spawn = shooterAngle(Shot, dx, dy, 0, Angle);
   bindentity(Spawn, self, dx, dz, dy, Dir, Flag);
}

void shooterRand(void Shot, float dx, float dy, float V, int Angle, int RA)
{ // Shooting special projectile with speed control at defined Angle + random deviation
// Shot : Projectile name
// dx : x offset
// dy : y offset
// V : Shoot speed
// Angle : Shooting angle
// RA : random deviation angle
    int r;

    if(RA == 0){
      r = 0;
    } else {
      r = rand()%RA;
    }

    shooterAngle(Shot, dx, dy, V, Angle+r);
}

void Rain(void Shot, float dx, float dy, float V, int Angle)
{ // Drops projectile relative to screen with speed control at defined Angle
    void self = getlocalvar("self");
    void vSpawn; //Spawn object.
    int Direction = getentityproperty(self, "direction");
    int XPos = openborvariant("xpos"); //Get screen edge's position
    int Screen = openborvariant("hResolution"); // Get screen width

    if(Direction == 0){ //Is entity facing left?                  
      dx = Screen-dx; //Reverse X direction to match facing and screen length
    }

    vSpawn = shooterAngle(Shot, 0, 0, V, Angle);

    changeentityproperty(vSpawn, "position", dx + XPos, NULL(), dy);   
}

void RandoRain(void Shot, float dy, float V)
{ // Drops projectile at random x position with speed control
    void self = getlocalvar("self");
    int Direction = getentityproperty(self, "direction");
    int XPos = openborvariant("xpos"); //Get screen edge's position
    int Screen = openborvariant("hResolution"); // Get screen width
    int r = rand()%40 + 50;
    void vSpawn; //Spawn object.
    int x;

    vSpawn = shooterAngle(Shot, 0, 0, V, -90);
    x = r*Screen*0.01;

    changeentityproperty(vSpawn, "position", x + XPos, NULL(), dy);
}
